default_app_config = 'waldur_mastermind.marketplace_openstack.apps.MarketplaceOpenStackConfig'
PACKAGE_TYPE = 'Packages.Template'
INSTANCE_TYPE = 'OpenStackTenant.Instance'
VOLUME_TYPE = 'OpenStackTenant.Volume'
